@@ -1,32 +1,9 @@
 # Protocol Buffers - Google's data interchange format
 # Copyright 2008 Google Inc.  All rights reserved.
-# https://developers.google.com/protocol-buffers/
 #
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are
-# met:
-#
-#     * Redistributions of source code must retain the above copyright
-# notice, this list of conditions and the following disclaimer.
-#     * Redistributions in binary form must reproduce the above
-# copyright notice, this list of conditions and the following disclaimer
-# in the documentation and/or other materials provided with the
-# distribution.
-#     * Neither the name of Google Inc. nor the names of its
-# contributors may be used to endorse or promote products derived from
-# this software without specific prior written permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+# Use of this source code is governed by a BSD-style
+# license that can be found in the LICENSE file or at
+# https://developers.google.com/open-source/licenses/bsd
 
 """Determine which implementation of the protobuf API is used in this process.
 """
@@ -36,6 +13,8 @@ import os
 import sys
 import warnings
 
+_GOOGLE3_PYTHON_UPB_DEFAULT = True
+
 
 def _ApiVersionToImplementationType(api_version):
   if api_version == 2:
@@ -102,10 +81,11 @@ if _implementation_type == 'cpp':
   try:
     # pylint: disable=g-import-not-at-top
     from google.protobuf.pyext import _message
+    sys.modules['google3.net.proto2.python.internal.cpp._message'] = _message
     _c_module = _message
     del _message
   except ImportError:
-    # TODO(jieluo): fail back to python
+    # TODO: fail back to python
     warnings.warn(
         'Selected implementation cpp is not available.')
     pass
@@ -151,14 +131,8 @@ def Type():
   return _implementation_type
 
 
-def _SetType(implementation_type):
-  """Never use! Only for protobuf benchmark."""
-  global _implementation_type
-  _implementation_type = implementation_type
-
-
 # See comment on 'Type' above.
-# TODO(jieluo): Remove the API, it returns a constant. b/228102101
+# TODO: Remove the API, it returns a constant. b/228102101
 def Version():
   return 2
 
